@@ -157,9 +157,8 @@
 #' @import mathjaxr
 #' @importFrom splines bs ns
 #'
-#' @docType package
 #' @name JointAI
-NULL
+"_PACKAGE"
 
 
 #' Create a Survival Object
@@ -174,7 +173,6 @@ NULL
 Surv <- survival::Surv
 
 
-
 #' Generate a Basis Matrix for Natural Cubic Splines
 #'
 #' This function just calls \code{ns()} from the
@@ -198,18 +196,28 @@ ns <- splines::ns
 #' @keywords internal
 # bs <- splines::bs
 
-bs <- function(x, df = NULL, knots = NULL, degree = 3, intercept = FALSE,
-               Boundary.knots = range(x), warn.outside = TRUE) {
-
+bs <- function(
+  x,
+  df = NULL,
+  knots = NULL,
+  degree = 3,
+  intercept = FALSE,
+  Boundary.knots = range(x),
+  warn.outside = TRUE
+) {
   defargs <- formals(splines::bs)
-  args <- sapply(names(defargs), function(k)
-    get(k), simplify = FALSE)
+  args <- sapply(
+    names(defargs),
+    function(k) {
+      get(k)
+    },
+    simplify = FALSE
+  )
 
   do.call(splines::bs, args)
 }
 
 
-
 .onLoad <- function(libname, pkgname) {
   rjags::load.module("glm", quiet = TRUE)
 }
@@ -221,7 +229,7 @@ bs <- function(x, df = NULL, knots = NULL, degree = 3, intercept = FALSE,
 }
 
 
-utils::globalVariables(c("i", "value", "chain", "iteration"))
+utils::globalVariables(c("i", "value", "chain", "iteration", "L1"))
 
 
 #' Parameters used by several functions in JointAI
@@ -302,4 +310,3 @@ utils::globalVariables(c("i", "value", "chain", "iteration"))
 #'                names that belong to the respective structure.
 #' @name sharedParams
 NULL
-
