@@ -91,7 +91,10 @@
 #'            \item{`append_data_list`}{list that will be appended to the list
 #'                 containing the data that is passed to **rjags**
 #'                 (`data_list`). This may be necessary if additional data /
-#'                 variables are needed for custom (covariate) models.}
+#'                 variables are needed for custom (covariate) models. Note:
+#'                 since version 1.0.7 elements of `append_data_list` will
+#'                 overwrite existing elements of the `data_list` with the same
+#'                 name.}
 #'            \item{`progress.bar`}{character string specifying the type of
 #'                 progress bar. Possible values are "text" (default), "gui",
 #'                 and "none" (see \code{\link[rjags]{update}}). Note: when
@@ -726,12 +729,14 @@ model_imp <- function(formula = NULL, fixed = NULL, data, random = NULL,
     random <- split_formula_list(formula)$random
   }
 
-  # check if the arguments meth, n.cores or parallel are provided
-  # (no longer used)
+
+  # return an error if the deprecated arguments meth, n.cores or parallel are
+  # provided
   args <- as.list(match.call())
+
   if (!is.null(args$meth))
     errormsg("The argument %s has been replaced by the argument %s.",
-              dQuote("meth"), dQuote("models"))
+             dQuote("meth"), dQuote("models"))
 
   if (!is.null(args$parallel) | !is.null(args$n.cores)) {
     errormsg("The arguments %s and %s are no longer used. To perform the
@@ -742,7 +747,7 @@ model_imp <- function(formula = NULL, fixed = NULL, data, random = NULL,
 
 
   # data pre-processing --------------------------------------------------------
-  data <- check_data(data, fixed, random, auxvars, timevar, mess)
+  data <- check_data(data, fixed, random, auxvars, timevar, mess, warn)
 
 
   # * divide matrices ----------------------------------------------------------
@@ -757,11 +762,11 @@ model_imp <- function(formula = NULL, fixed = NULL, data, random = NULL,
 
   # * model dimensions ---------------------------------------------------------
   par_index_main <- get_model_dim(Mlist$lp_cols[names(Mlist$lp_cols) %in%
-                                     names(Mlist$fixed)],
-                     Mlist = Mlist)
+                                                  names(Mlist$fixed)],
+                                  Mlist = Mlist)
   par_index_other <- get_model_dim(Mlist$lp_cols[!names(Mlist$lp_cols) %in%
-                                         names(Mlist$fixed)],
-                         Mlist = Mlist)
+                                                   names(Mlist$fixed)],
+                                   Mlist = Mlist)
 
   # * model info ---------------------------------------------------------------
   info_list <- get_model_info(Mlist, par_index_main = par_index_main,
